﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("ActionExecuter", "")]
  [StorableClass]
  public class ActionExecuter : SingleSuccessorOperator, IActionExecuter {

    #region parameter
    public IValueLookupParameter<IAction> SelectedActionParameter {
      get { return (IValueLookupParameter<IAction>)Parameters["SelectedAction"]; }
    }
    public IValueLookupParameter<IAction> CurrentActionToMatchParameter {
      get { return (IValueLookupParameter<IAction>)Parameters["CurrentActionToMatch"]; }
    }
    public IValueLookupParameter<DoubleValue> CurrentPayoffParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["CurrentPayoff"]; }
    }
    public ILookupParameter<DoubleValue> NegativeRewardParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["NegativeReward"]; }
    }
    public ILookupParameter<DoubleValue> PositiveRewardParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["PositiveReward"]; }
    }
    #endregion

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ActionExecuter(this, cloner);
    }
    protected ActionExecuter(ActionExecuter original, Cloner cloner)
      : base(original, cloner) {
    }
    [StorableConstructor]
    protected ActionExecuter(bool deserializing) : base(deserializing) { }

    public ActionExecuter()
      : base() {
      Parameters.Add(new ValueLookupParameter<IAction>("SelectedAction"));
      Parameters.Add(new ValueLookupParameter<IAction>("CurrentActionToMatch"));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("CurrentPayoff"));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("NegativeReward"));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("PositiveReward"));
    }

    public override IOperation Apply() {
      if (SelectedActionParameter.ActualValue.Match(CurrentActionToMatchParameter.ActualValue)) {
        CurrentPayoffParameter.ActualValue = PositiveRewardParameter.ActualValue;
      } else {
        CurrentPayoffParameter.ActualValue = NegativeRewardParameter.ActualValue;
      }
      return base.Apply();
    }
  }
}
